importPackage(java.awt);
importPackage(java.awt.geom);

var useEnglishText = false;

include(Resources.id("mtrsteamloco:scripts/display_helper.js"));

let slotCfg = {
	"version": 1,
	"texSize": [640, 128],
	"slots": [
		{
			"name": "front",    
			"texArea": [0, 0, 640, 128],
			"pos": [
				[[-0.6701, 2.183, 7.610],  [-0.6701, 1.916, 7.610], 
				[0.6776, 1.916, 7.610], [0.6776, 2.183, 7.610]]
			],
			"offsets": [[0, 0, 0]]
		}
	]
};

var font1 = Resources.readFont(Resources.idRelative("mtr:font/notosans.ttf"));
var font2 = Resources.readFont(Resources.idRelative("mtr:font/notosans_light.ttf"));
const maxTickDispConst = 80;
var dhBase = new DisplayHelper(slotCfg);

function create(ctx, state, train) {
	state.dh = dhBase.create();
	state.rate = new RateLimit(1.0);

	drawDispl(state, train)
}

function dispose(ctx, state, train) {
	if (state.dh != null) {
		state.dh.close();
	}
}

function render(ctx, state, train) {
	//renderDispl(ctx, state, train); // неиспользуется из-за того, что вагон качается, а дисплей нет.
	updateDispl(ctx, state, train);
}

function renderDispl(ctx, state, train){
	let matrices = new Matrices();
	matrices.pushPose();
	ctx.drawCarModel(state.dh.model, 0, matrices);
	matrices.rotateY(3.14159);
	ctx.drawCarModel(state.dh.model, train.trainCars() - 1, matrices);
	matrices.popPose();
}

function updateDispl(ctx, state, train){
	if (state.rate.shouldUpdate()){
		drawDispl(state, train);
	}
}

function drawDispl(state, train){

	let routeNum = "--"
	let endStation = "Nicht im Dienst"
	let color = Color.BLACK;

	if (train.getThisRoutePlatforms().size() != 0) {
		routeNum = getRouteNum(train).substring(0, 3);
		endStation = processName(getEndStation(train))
		color = getRouteColor(train);
	}

	let g = state.dh.graphics();

	const xNum = 80;
	const xDest = 380;
	const yNum = 100;
	const yDest = 90;
	//filling
	g.setColor(Color.BLACK);
	g.fillRect(0, 0, 640, 128);
	g.setColor(Color.WHITE);

	// route destination
	g.setFont(font1.deriveFont(0, 62))
	drawStringCenter(g, endStation, xDest, yDest);
	
	// filling with route color
	g.setColor(new Color(color.r / 255, color.g / 255, color.b / 255));
	g.fillRect(0, 0, 152, 128);

	// route number
	g.setColor(Color.WHITE);
	g.setFont(font2.deriveFont(1, 80))
	drawStringCenter(g, routeNum, xNum, yNum);

	state.dh.upload();
}

function processName(a) {
	let aJsStr = "" + a;
	if (a.includes('|')) {

		let strs = aJsStr.split('|')
		aJsStr = useEnglishText ? strs[1] : strs[0];
	}
	return aJsStr;
}


function clamp(num, min, max) {
  return Math.min(Math.max(num, min), max);
} 

function getRouteNum(train) {
	let stationList = train.getThisRoutePlatforms();
	let ret = stationList[0].route.lightRailRouteNumber + "";
	return ret;
}

function getRouteColor(train) {
	let stationList = train.getThisRoutePlatforms();
	let ret = stationList[0].route.color;
	return decodeColor(ret);
}

function decodeColor(color) {
  let r = (color >> 16) & 0xFF;  // Извлекаем старший байт для красного
  let g = (color >> 8) & 0xFF;   // Извлекаем средний байт для зеленого
  let b = color & 0xFF;          // Извлекаем младший байт для синего
  return { r, g, b };
}

function getEndStation(train) {
  let stationList = train.getThisRoutePlatforms();
  
  let regularEndStation = stationList[0].destinationName;

  let lastSt = stationList.size() - 1;
  let customEndStation = stationList[lastSt].route.platformIds[lastSt].customDestination + "";

  return customEndStation == "" ? regularEndStation : customEndStation;
}

function drawStringCenter(g, string, x, y){
  let fontMetrics = g.getFontMetrics();
  const newX = x - fontMetrics.stringWidth(string) / 2;

  g.drawString(string, newX, y)
}
